--
-- Mod: Animal Worker
--
-- Author: Tunnelrat
-- @Date: 06.25.2019
-- @Version: 2.1 Animal worker automatically tends to animal needs for a daily fee.
-- Copyright (C) Tunnelrat, All Rights Reserved.

animalWorker = {};

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml")

animalWorkerActive = true -- if true, the Ranch Hand will work everyday, set to false to disable. Default: true
animalWorkerFee = 0.05 -- convenience fee percentage for doing all this work. Default: 0.05
costPerAnimal = 10.00 -- base cost to tend to each animal. Default: 10.00
fillToCapacity = true -- false if you want to give enough food,water,straw for 6 days. Default: true
cleanPens = true -- if true, will also clean the animal pens once a day. Default: true

animalWorker.version = getXMLString(modDesc, "modDesc.version")
animalWorker.modDirectory = g_currentModDirectory

addModEventListener(animalWorker)

function animalWorker:loadMap()

print("############################################################")
print("--- Animal Worker v.:" .. self.version .. ", by Tunnelrat.")
print("############################################################")

-- Settings
animalWorker.settingsDir = getUserProfileAppPath().. "modsSettings/animalWorker/"

animalWorker.LoadSettings()
animalWorker.SaveSettings()

end

function animalWorker:SaveSettings()

	createFolder(getUserProfileAppPath() .. "modsSettings/")
	createFolder(animalWorker.settingsDir)
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	local pos = 0
	local rootPath = "animalWorker.settings"
	local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)

	if not fileExists(file) then
		-- create
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapaicity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		saveXMLFile(xmlContent)
	else
		-- update
		local xmlContent = createXMLFile("animalWorker_XML", file, "animalWorker")
		
		setXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive", animalWorkerActive)
		setXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee", animalWorkerFee)
		setXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal", costPerAnimal)
		setXMLBool(xmlContent, groupNameTag .. "#fillToCapaicity", fillToCapacity)
		setXMLBool(xmlContent, groupNameTag .. "#cleanPens", cleanPens)
		saveXMLFile(xmlContent)
	end

end

function animalWorker:LoadSettings()
	
	local file = animalWorker.settingsDir .. "animalWorker.xml"
	
	if fileExists(file) then
		local xmlContent = loadXMLFile("animalWorker_XML", file, "animalWorker")
		
		local pos = 0
		local rootPath = "animalWorker.settings"
		local groupNameTag = string.format("%s.setting(%d)", rootPath, pos)
		
		animalWorkerActive 	= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#animalWorkerActive"), true)
		animalWorkerFee		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#animalWorkerFee"), 0.05)
		costPerAnimal		= Utils.getNoNil(getXMLFloat(xmlContent, groupNameTag .. "#costPerAnimal"), 10.00)
		fillToCapacity		= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#fillToCapaicity"), true)
		cleanPens			= Utils.getNoNil(getXMLBool(xmlContent, groupNameTag .. "#cleanPens"), true)
	else
		print("Animal Worker XML settings not found.")
	end
	
end

function animalWorker:keyEvent(unicode, sym, modifier, isDown)

end

function animalWorker:mouseEvent(posX, posY, isDown, isUp, button)
end


function animalWorker:deleteMap()
end

function animalWorker:update(dt)

	local hourNow = g_currentMission.environment.currentHour
	local minuteNow = g_currentMission.environment.currentMinute
	--local stats = g_currentMission:farmStats(self:getOwnerFarmId())
	--print("animalWorker:debug Time: " .. hourNow .. " Minute: " .. minuteNow)
	
	farmData = {
		ownerFarmId = 0,
		farmId = 0,
	}
	
	local playerOwnsFarm = false
	
	function checkIsOwnerOfFarm(husbandry)
		if g_currentMission:getIsClient() then
			farmData.ownerFarmId = g_currentMission.player.ownerFarmId
			farmData.farmId = g_currentMission.player.farmId
			
			if husbandry.modulesByName.animals.owner.ownerFarmId == farmData.farmId then
				playerOwnsFarm = true
				return true
			else
				playerOwnsFarm = false
				return false
			end
		end
	end
	
	function animalCount(husbandry)
		if husbandry ~= nil then
			local animals = husbandry.modulesByName.animals.animals
			local numAnimals = 0
			for k,v in pairs(animals) do
				numAnimals = tonumber(k)
			end
			return numAnimals
		else
			print("animalWorker:bad parameters for animalCount()")
			return 0
		end
	end
	--[[FillTypes concatenated
	local wheat = 2	local barley = 3	local canola = 5	local forage = 19	local silage = 24	local hay = 30
	local grass = 28	local sunflower = 6	local soybean = 7 -- g_currentMission.fillTypeManager:getFillTypeIndexByName("fillType_soybean")
	local maize = 8	local oat = 4	local potato = 9	local sugerbeet = 10]]--
	local chickens, cows, horse, pigs, sheep, all = 0, 0, 0, 0, 0, 0
	local eatDayChicken = 5
	local eatDayCow = 350
	local eatDayHorse = 400
	local eatDayPig = 90
	local eatDaySheep = 50
	
	-- Fill water troughs, food and straw at 7am if worker is active
	if hourNow == 7 and minuteNow == 00 and animalWorkerActive == true then
		
		-- CHICKENS
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
				
				if animalType ~= nil and animalType == "CHICKEN" then
					chickens = animalCount(husbandry)
					
					-- Clean Pens
					if cleanPens == true then
						if husbandry.modulesByName.foodSpillage ~= nil then
							husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
							husbandry:setModuleParameters("foodSpillage", 0, 0)
						end
					end
					
					-- Water
					if husbandry.modulesByName.water ~= nil then
						local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
						capacity = husbandry.modulesByName.water.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
							else
								local fillTo = tonumber(math.ceil(chickens * 7 * 6)) -- 7 L per chicken (6 days)
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
							end
						end
					end
				
					-- Food					
					if chickens > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(chickens * 5 * 6)) -- numAnimals * litersFoodPerAnimal * dayToInterval
						else
							foodToAdd = tonumber(fCap)
						end
						print("animalWorker: Fed " .. foodToAdd .. " wheat and barley to " .. chickens .. " chickens.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, foodToAdd)
						end
					end
				end	-- End feeding
			
			end -- owner
		
		end -- for loop chicken
		
		-- COWS
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
			
				if animalType ~= nil and animalType == "COW" then
					cows = animalCount(husbandry)
					
					-- Clean Pens
					if cleanPens == true then
						if husbandry.modulesByName.foodSpillage ~= nil then
							husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
							husbandry:setModuleParameters("foodSpillage", 0, 0)
						end
					end
					
					-- Water
					if husbandry.modulesByName.water ~= nil then
						local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
						capacity = husbandry.modulesByName.water.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
							else
								local fillTo = tonumber(math.ceil(cows * 80 * 6)) -- 80 L per cow (6 days)
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
							end
						end
					end
			
					-- Straw
					if husbandry.modulesByName.straw ~= nil then
						local amountToAdd = husbandry.modulesByName.straw:getFreeCapacity(FillType.STRAW)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.straw:getFillLevel(FillType.STRAW)
						capacity = husbandry.modulesByName.straw.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)
							else
								local fillTo = tonumber(math.ceil(cows * 95 * 6)) -- 95 L per cow
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, fillTo)
							end
						end
					end
				
					-- Food
					if cows > 0 then
						local foodToAdd, f75, f25 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(cows * 500 * 6))
							f75 = tonumber(math.floor(cows * math.ceil(350 * 0.75) * 6)) -- 75% eat weight
							f25 = tonumber(math.floor(cows * math.ceil(350 * 0.25) * 6)) -- 25%
						else
							f75 = tonumber(math.ceil(fCap * 0.75))
							f25 = tonumber(math.ceil(fCap * 0.25))
							foodToAdd = tonumber(fCap)
						end
						print("animalWorker: Fed " .. foodToAdd .. " TMR, " .. f75 .. " silage and hay, " .. f25 .. " grass to " .. cows .. " cows.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.FORAGE, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.SILAGE, f75)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, f75)
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, f25)
						end
					end
				end
			end
		end -- for loop cow
		
		-- HORSE
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
			
				if animalType ~= nil and animalType == "HORSE" then
					horse = animalCount(husbandry)
					
					-- Clean Pens
					if cleanPens == true then
						if husbandry.modulesByName.foodSpillage ~= nil then
							husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
							husbandry:setModuleParameters("foodSpillage", 0, 0)
						end
					end
					
					-- Water
					if husbandry.modulesByName.water ~= nil then
						local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
						capacity = husbandry.modulesByName.water.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
							else
								local fillTo = tonumber(math.ceil(cows * 50 * 6)) -- 50 L per horse (6 days)
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
							end
						end
					end
			
					-- Straw
					if husbandry.modulesByName.straw ~= nil then
						local amountToAdd = husbandry.modulesByName.straw:getFreeCapacity(FillType.STRAW)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.straw:getFillLevel(FillType.STRAW)
						capacity = husbandry.modulesByName.straw.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)
							else
								local fillTo = tonumber(math.ceil(cows * 80 * 6)) -- 80 L per horse
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, fillTo)
							end
						end
					end
				
					-- Food					
					if horse > 0 then
						local foodToAdd, f60, f40 = 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(horse * 400 * 6))
							f60 = tonumber(math.floor(horse * math.ceil(400 * 0.6) * 6)) -- 60% eat weight
							f40 = tonumber(math.floor(horse * math.ceil(400 * 0.4) * 6)) -- 40% eat weight
						else
							f60 = tonumber(math.ceil(fCap * 0.6))
							f40 = tonumber(math.ceil(fCap * 0.4))
							foodToAdd = tonumber(fCap)
						end
						print("animalWorker: Fed " .. foodToAdd .. " oats and hay to " .. horse .. " horses.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.OAT, foodToAdd)
							husbandry.modulesByName.food:setFillLevel(FillType.DRYGRASS_WINDROW, foodToAdd)
						end
					end
				end
			end
		end -- for loop horse
		
		-- PIG
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
			
				if animalType ~= nil and animalType == "PIG" then
					pigs = animalCount(husbandry)
				
					-- Clean Pens
					if cleanPens == true then
						if husbandry.modulesByName.foodSpillage ~= nil then
							husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
							husbandry:setModuleParameters("foodSpillage", 0, 0)
						end
					end
					
					-- Water
					if husbandry.modulesByName.water ~= nil then
						local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
						capacity = husbandry.modulesByName.water.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
							else
								local fillTo = tonumber(math.ceil(cows * 20 * 6)) -- 20 L per pig (6 days)
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
							end
						end
					end
			
					-- Straw
					if husbandry.modulesByName.straw ~= nil then
						local amountToAdd = husbandry.modulesByName.straw:getFreeCapacity(FillType.STRAW)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.straw:getFillLevel(FillType.STRAW)
						capacity = husbandry.modulesByName.straw.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, capacity)
							else
								local fillTo = tonumber(math.ceil(cows * 30 * 6)) -- 30 L per pig
								husbandry.modulesByName.straw:setFillLevel(FillType.STRAW, fillTo)
							end
						end
					end
				
					-- Food
					if pigs > 0 then
						local foodToAdd, f50, f25, f20, f5 = 0, 0, 0, 0, 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(pigs * 90 * 6)) -- 100%
							f50 = tonumber(math.floor(pigs * math.ceil(90 * 0.5) * 6)) -- 50%
							f25 = tonumber(math.floor(pigs * math.ceil(90 * 0.25) * 6)) -- 25%
							f20 = tonumber(math.floor(pigs * math.ceil(90 * 0.2) * 6)) -- 20%
							f5 = tonumber(math.floor(pigs * math.ceil(90 * 0.05) * 6)) -- 5%
						else
							f50 = tonumber(math.ceil(fCap * 0.5))
							f25 = tonumber(math.ceil(fCap * 0.25))
							f20 = tonumber(math.ceil(fCap * 0.2))
							f5 = tonumber(math.ceil(fCap * 0.05))
							foodToAdd = tonumber(fCap)
						end
						print("animalWorker: Fed (" .. foodToAdd .. " total) " .. f50 .. " corn, " .. f25 .. " grains, " .. f20 .. " proteins, " .. f5 .. " roots to " .. pigs .. " pigs.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.MAIZE, f50)
							husbandry.modulesByName.food:setFillLevel(FillType.WHEAT, f25)
							husbandry.modulesByName.food:setFillLevel(FillType.BARLEY, f25)
							husbandry.modulesByName.food:setFillLevel(FillType.SOYBEAN, f20)
							husbandry.modulesByName.food:setFillLevel(FillType.CANOLA, f20)
							husbandry.modulesByName.food:setFillLevel(FillType.SUNFLOWER, f20)
							husbandry.modulesByName.food:setFillLevel(FillType.POTATO, f5)
							husbandry.modulesByName.food:setFillLevel(FillType.SUGARBEET, f5)
						end
					end
				end
			end
		end -- for loop pig
		
		-- SHEEP
		for _,husbandry in pairs(g_currentMission.husbandries) do
		
			local animalType = husbandry.modulesByName.animals.animalType
		
			if checkIsOwnerOfFarm(husbandry) == true then
			
				if animalType ~= nil and animalType == "SHEEP" then
					sheep = animalCount(husbandry)
					
					-- Clean Pens
					if cleanPens == true then
						if husbandry.modulesByName.foodSpillage ~= nil then
							husbandry.modulesByName.foodSpillage.cleanlinessFactor = 1.0
							husbandry:setModuleParameters("foodSpillage", 0, 0)
						end
					end
					
					-- Water
					if husbandry.modulesByName.water ~= nil then
						local amountToAdd = husbandry.modulesByName.water:getFreeCapacity(FillType.WATER)
						local fillLevel, capacity = 0, 0
    
						fillLevel = husbandry.modulesByName.water:getFillLevel(FillType.WATER)
						capacity = husbandry.modulesByName.water.fillCapacity
		
						if amountToAdd > 0 then
							if fillToCapacity == true then
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, capacity)					
							else
								local fillTo = tonumber(math.ceil(cows * 20 * 6)) -- 20 L per sheep (6 days)
								husbandry.modulesByName.water:setFillLevel(FillType.WATER, fillTo)
							end
						end
					end
			
					-- Food					
					if sheep > 0 then
						local foodToAdd = 0
						local fCap = husbandry.modulesByName.food:getCapacity()
						if fillToCapacity == false then
							foodToAdd = tonumber(math.floor(sheep * 50 * 6)) -- 100%
						else
							foodToAdd = tonumber(fCap)
						end
						print("animalWorker: Fed " .. foodToAdd .. " grass to " .. sheep .. " sheep.")
						if foodToAdd > 0 then
							husbandry.modulesByName.food:setFillLevel(FillType.GRASS_WINDROW, foodToAdd)
						end
					end
				end
			end
		end -- for loop sheep
		
		all = tonumber(chickens + cows + pigs + horse + sheep)
		
		-- Pay the worker...
		local difficultyMultiplier = g_currentMission.missionInfo.buyPriceMultiplier
		-- local waterCost = g_currentMission.economyManager:getCostPerLiter(FillType.WATER, false)
		local moneyUnit = g_gameSettings.moneyUnit
		local showCurrency = "DOLLARS"
		if moneyUnit == "1" then
			showCurrency = "EUROS"
		elseif moneyUnit == "2" then
			showCurrency = "DOLLARS"
		elseif moneyUnit == "3" then
			showCurrency = "POUNDS"
		end
		local price = 0
		local farmCost = 0
		if all ~= 0 and minuteNow == 00 then
			price = tonumber(math.ceil(costPerAnimal * all * difficultyMultiplier)) -- 10 * numAnimals * {1.0, 1.8, 3.0}
			price = price + (math.floor(price * animalWorkerFee)) -- sum with fee
			farmCost = tonumber(math.floor(price / 60)) -- per second in one minute (needs to be optimized)
			print("animalWorker:Cost " .. farmCost .. " total.")
			if farmCost ~= 0 then
				g_currentMission:farmStats(farmData.farmId):updateStats("expenses", farmCost)
				g_currentMission:addMoney(-farmCost, farmData.farmId, MoneyType.ANIMAL_UPKEEP, true)
				print("animalWorker:Ranch hand fed, watered and bedded all " .. all .. " animals at 7am for " .. farmCost .. " " .. showCurrency .. ".")
			end
		end	-- End costs
		
	end
end

function animalWorker:draw()
end